/*
 * daemon watcher (dw)
 * ATM  11/12/91
 */
#include <stdio.h>
#include <signal.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "defs.h"

#define LOGFILE	"/usr/local/games/lib/netrek/dwlog"
#define PROG	"/usr/local/games/lib/netrek/daemonII.real"

static int Restart = 0;
FILE *logfp;

main(argc, argv)
int argc;
char **argv;
{
    pid_t fpid, wpid;
    int stat;
    time_t now;

    if ((logfp = fopen(LOGFILE, "a")) == NULL) {
	perror("fopen");
	fprintf(stderr, "Error: unable to open %s\n", LOGFILE);
	exit(1);
    }
    setvbuf(logfp, (char *)NULL, _IONBF, 0);	/* unbuffered stream */

    if ((argc > 1) && !strcmp(argv[1], "-R"))
	Restart = 1;

restart_daemon:
    if (!(fpid = fork())) {
	if (!Restart)
	    execl(PROG, "daemonII (real)", NULL);
	else
	    execl(PROG, "daemonII (real)", "-R", NULL);
	fprintf(stderr, "dw failed!!  daemonII not started!!  ");
	fprintf(logfp, "dw exec failed!!  daemonII not started!!");
	perror("exec");
	exit(1);
    }

    signal(SIGINT, SIG_IGN);
    signal(SIGHUP, SIG_IGN);
/*    fprintf(stderr, "true daemonII is %d\n", fpid);*/

    now = time(0);
    fprintf(logfp, "\nDaemon (%d) started at %s", fpid, ctime(&now));
    Restart = 1;

    if ((wpid = wait(&stat)) < 0) {
	perror("wait");
	exit(1);
    }
/*    fprintf(stderr, "dw reaped %d, status is 0x%x\n", wpid, stat);*/
    fprintf(logfp, "dw reaped %d, status is 0x%x\n", wpid, stat);
    if (stat == SIGALRM) {
/*	fprintf(stderr, "Stray SIGALRM, restarting.\n");*/
	fprintf(logfp, "Stray SIGALRM, restarting.");
	goto restart_daemon;
    }
    if (!(stat & 0xffffff00) && (stat & 0x000000ff)) {
	fprintf(stderr, "Hey!  daemonII was killed by a signal!\n");
	fprintf(logfp, "Hey!  daemonII was killed by a signal!\n");
	if (ask_restart())
	    exit(1);
	else {
/*	    fprintf(stderr, "Restarting daemonII.");*/
	    goto restart_daemon;
	}
    }
    if ((stat & 0xffffff00) && !(stat & 0x000000ff)) {
	fprintf(stderr, "Hey!  daemonII exited abnormally!\n");
	fprintf(logfp, "Hey!  daemonII exited abnormally!\n");
	exit(1);
    }

/*    fprintf(stderr, "daemonII exited normally.\n");*/
/*    fprintf(logfp, "daemonII exited normally.\n");*/
    exit(0);
    /*NOTREACHED*/
}


/*
 * Returns 0 on successful restart, 1 otherwise.
 */
int
ask_restart()
{
    FILE *fp;
    char buf[100];
    int alarm_handler();

    if ((fp = fopen("/dev/tty", "r+")) == NULL)
	return(1);

    signal(SIGALRM, alarm_handler);
    alarm(30);

    fprintf(fp,
	"Should I (r)estart, (p)urge shared mem & exit, or just (q)uit? ");
    fflush(fp);
    fgets(buf, 100, fp);

    alarm(0);
    signal(SIGALRM, SIG_DFL);

    switch (buf[0]) {
    case 'p':
	purge();
	return(1);

    case 'q':
	return(1);

    case 'r':
    default:
	return(0);
    }

}

int
purge()
{
    extern int errno;
    int shmemKey = PKEY;
    int shmid;

    errno = 0;
    shmid = shmget(shmemKey, 0, 0);
    if (shmid < 0) {
	if (errno != ENOENT) {
	    perror("shmget");
	    return(1);
	}
	shmid = shmget(shmemKey, 0, 0);
	if (shmid < 0) {
	    fprintf(stderr, "segment already killed\n");
	    return(1);
	}
    }

    shmctl(shmid, IPC_RMID, 0);
}


int
alarm_handler(sig)
int sig;
{
    fprintf(stderr, "dw timed out, purging segment and exiting\n");
    fprintf(logfp, "dw timed out, purging segment and exiting\n");
    purge();
    exit(1);
}

