/*
 * redraw.c
 */
#include "copyright.h"

#include <stdio.h>
#include <signal.h>
#include <math.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "packets.h"

static short nplayers;

intrupt()
{
    if (!binconfirm) testtime=0;
    if (testtime == -1) {
	struct reserved_spacket sp;

	    /* Give reasonable period of time to respond to query 
	     *  (and test code if they need to) 
	     */
	testtime = 200;	
	makeReservedPacket(&sp);
	bcopy(sp.data, testdata, 16);
	sendClientPacket(&sp);
    } else if (testtime != 0) {
	testtime--;
	if (testtime==0) {
	    /* User failed to respond to verification query.  Bye! */
	    me->p_explode = 10;
	    me->p_whydead = KQUIT;
	    me->p_status = PEXPLODE;
	    fprintf(stderr, "User binary failed to verify\n");
/*	    exit(0);*/		/* ATM: why exit now? */
	}
    }
    if (me->p_status == PFREE) {
	me->p_ghostbuster = 0;
	me->p_status = PDEAD;
    }
    if (me->p_status == PEXPLODE || me->p_status == PDEAD) {
	inputMask=0;
    }
    if (((me->p_status == PDEAD) || (me->p_status == POUTFIT))
	&& (me->p_ntorp <= 0) && (me->p_nplasmatorp <= 0)) {
	death();
    }

    auto_features();
    updateClient();
}

/* These are routines that need to be done on interrupts but
   don't belong in the redraw routine and particularly don't
   belong in the daemon. */

auto_features()
{
    char buf[180];
    struct player *pl;
    struct planet *pln;
    unsigned char course;
    int troop_capacity=0;

    if (me->p_flags & PFSELFDEST) {
	if ((me->p_updates >= selfdest) ||
	    ((me->p_flags & PFGREEN) && (me->p_damage == 0)
		&& (me->p_shield == me->p_ship.s_maxshield))) {
	    me->p_flags &= ~PFSELFDEST;
	    me->p_explode = 10;
	    me->p_whydead = KQUIT;
	    me->p_status = PEXPLODE;
	} else {
	    switch ((selfdest - me->p_updates)/10) {
	    case 5: case 4:
		warning("You notice everyone on the bridge is staring at you.");
		break;
	    default:
		sprintf(buf, "Stand By ... Self Destruct in %d seconds",
			(selfdest - me->p_updates) / 10);
		warning(buf);
		break;
	    }
	}
    }
    
    /* give certain information about bombing or beaming */
    if (me->p_flags & PFBOMB) {
	if (planets[me->p_planet].pl_armies < 5) {
	    sprintf(buf, "Weapons Officer: Bombing is ineffective when less than 5 armies are defending.");
	    warning(buf);
	    me->p_flags &= ~PFBOMB;
	}
	else {
	    sprintf(buf, "Weapons Officer: Bombarding %s...  Sensors read %d armies left.",
		planets[me->p_planet].pl_name,
		planets[me->p_planet].pl_armies);
	    warning(buf);
	}
    }

    if (me->p_ship.s_type == ASSAULT)
	troop_capacity = (((me->p_kills * 3) > myship->s_maxarmies) ?
	    myship->s_maxarmies : (int) (me->p_kills * 3));
    else if (me->p_ship.s_type != STARBASE)
	troop_capacity = (((me->p_kills * 2) > myship->s_maxarmies) ?
	    myship->s_maxarmies : (int) (me->p_kills * 2));
    else
	troop_capacity = me->p_ship.s_maxarmies;

    if (me->p_flags & PFBEAMUP) {
	if (me->p_flags & PFORBIT) {
	    if (planets[me->p_planet].pl_armies < 5) {
		sprintf(buf, "%s: Too few armies to beam up",
			planets[me->p_planet].pl_name);
		warning(buf);
		me->p_flags &= ~PFBEAMUP;
	    } else if (me->p_armies == troop_capacity) {
		sprintf(buf, "No more room on board for armies");
		warning(buf);
		me->p_flags &= ~PFBEAMUP;
	    } else {
		sprintf(buf, "Beaming up.  (%d/%d)", me->p_armies, troop_capacity);
		warning(buf);
	    }
	} else if (me->p_flags & PFDOCK) {
	    if (players[me->p_docked].p_armies == 0) {
		sprintf(buf, "Starbase %s: Too few armies to beam up",
			players[me->p_docked].p_name);
		warning(buf);
		me->p_flags &= ~PFBEAMUP;
	    } else if (me->p_armies == troop_capacity) {
		sprintf(buf, "No more room on board for armies");
		warning(buf);
		me->p_flags &= ~PFBEAMUP;
	    } else {
		sprintf(buf, "Beaming up.  (%d/%d)", me->p_armies, troop_capacity);
		warning(buf);
	    }
	}
    }

    if (me->p_flags & PFBEAMDOWN) {
	if (me->p_armies == 0) {
	    sprintf(buf, "No more armies to beam down to %s.",
		planets[me->p_planet].pl_name);
	    warning(buf);
	    me->p_flags &= ~PFBEAMDOWN;
	} else if (me->p_flags & PFORBIT) {
	    sprintf(buf, "Beaming down.  (%d/%d) %s has %d armies left",
		me->p_armies,
		troop_capacity, 
		planets[me->p_planet].pl_name,
		planets[me->p_planet].pl_armies);
	    warning(buf);
	} else if (me->p_flags & PFDOCK) {
	    if (players[me->p_docked].p_armies ==
		players[me->p_docked].p_ship.s_maxarmies) {
		sprintf(buf, "Transporter Room:  Starbase %s reports all troop bunkers are full!", 
		    players[me->p_docked].p_name);
		warning(buf);
		me->p_flags &= ~PFBEAMDOWN;
	    } else {
		sprintf(buf, "Transfering ground units.  (%d/%d) Starbase %s has %d armies left",
		    me->p_armies,
		    troop_capacity, 
		    players[me->p_docked].p_name,
		    players[me->p_docked].p_armies);
		warning(buf);
	    }
	}
    }
    if (me->p_flags & PFREPAIR) {
	if ((me->p_damage == 0) && (me->p_shield == me->p_ship.s_maxshield))
	    me->p_flags &= ~PFREPAIR;
    }
    if (me->p_flags & PFPLOCK) { 	/* set course to player x */
	int dist;

	pl = &players[me->p_playerl];
	if (pl->p_status != PALIVE)
	    me->p_flags &= ~PFPLOCK;
	if (pl->p_ship.s_type == STARBASE) {
	    dist = hypot((double) (me->p_x - pl->p_x),
		(double) (me->p_y - pl->p_y));
	    if (dist-(DOCKDIST/2) < (11500 * me->p_speed * me->p_speed) /
		    me->p_ship.s_decint) {
		if (me->p_desspeed > 2) {
		    set_speed(me->p_desspeed-1);
		}
	    }
	    if ((dist < DOCKDIST) && (me->p_speed <= 2))  {
		me->p_flags &= ~PFPLOCK;
		orbit();
	    }
	}
	if (me->p_flags & PFPLOCK) {
	    course = newcourse(pl->p_x, pl->p_y);
	    set_course(course);
	}
    }
    if (me->p_flags & PFPLLOCK) { 	/* set course to planet x */
	int dist;

	pln = &planets[me->p_planet];
	dist = hypot((double) (me->p_x - pln->pl_x),
	    (double) (me->p_y - pln->pl_y));

	/* This is magic.  It should be based on defines, but it slows
	   the ship down to warp two an appropriate distance from the
	   planet for orbit */

	if (dist-(ORBDIST/2) < (11500 * me->p_speed * me->p_speed) /
	        me->p_ship.s_decint) {
	    if (me->p_desspeed > 2) {
		set_speed(2);
	    }
	}

	if ((dist < ENTORBDIST) && (me->p_speed <= 2))  {
	    me->p_flags &= ~PFPLLOCK;
	    orbit();
	}
	else {
	    course = newcourse(pln->pl_x, pln->pl_y);
	    set_course(course);
	}
    }
}

newcourse(x, y)
int x, y;
{
	return((unsigned char) (atan2((double) (x - me->p_x),
	    (double) (me->p_y - y)) / 3.14159 * 128.));
}

