#include <X11/Xlib.h>		/* ATM */
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/file.h>
#include <math.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <errno.h>
#include <pwd.h>
#include <string.h>
#include <ctype.h>
#include "defs.h"
#include "struct.h"

struct player *players;
struct player *me;
struct torp *torps;
struct ship *myship;
struct stats *mystats;
struct planet *planets;
struct status *status;
struct phaser *phasers;
struct message *messages;
struct mctl *mctl;
struct team *teams;

int mode=0;

openmem()
{
    extern int errno;
    int	shmemKey = PKEY;
    int	shmid;
    struct memory	*sharedMemory;

    errno = 0;
    shmid = shmget(shmemKey, 0, 0);
    if (shmid < 0) {
	if (errno != ENOENT) {
	    perror("shmget");
	    exit(1);
	}
	shmid = shmget(shmemKey, 0, 0);
	if (shmid < 0) {
	    fprintf(stderr, "Daemon not running (nobody playing)\n");
	    exit (1);
	}
    }
    sharedMemory = (struct memory *) shmat(shmid, 0, 0);
    if (sharedMemory == (struct memory *) -1) {
	perror("shared memory");
	exit (1);
    }
    players = sharedMemory->players;
    torps = sharedMemory->torps;
    planets = sharedMemory->planets;
    status = sharedMemory->status;
    phasers = sharedMemory->phasers;
    mctl = sharedMemory->mctl;
    messages = sharedMemory->messages;
    teams = sharedMemory->teams;
}
char *ships[] = {"SC", "DD", "CA", "BB", "AS", "SB", "??"};

static char *linestr[2] = {
    "<>==========================================================================<>",
    "<>=======================================================================<>"
};


main(argc, argv)
int argc;
char **argv;
{
    int i;
    char monitor[17];
    int teams[9];
    int wins[9], losses[9];

    /* ATM: don't let players view each other's important stats */
    if (argc>1) mode= *(argv[1]);
    if (getuid() != geteuid())
	switch (mode) {
	case 'd':
	    mode = '-';
	    break;
	}

    switch(mode) {
    case 'c':
	showcount();
	break;
    case 'd':
	puts(linestr[1]);
	printf("  Pl: Name             Login    Type  Kills Damage Shields Armies   Fuel\n");
	puts(linestr[1]);
	break;
    default:
	puts(linestr[0]);
	printf("  Pl: Name             Display          Type Bombing Planets Offense Defense\n");
	puts(linestr[0]);
	break;
    }

    openmem();
    for (i=0; i<9; i++) teams[i]=0;

    for (i=0; i<MAXPLAYER; i++) {
	if (players[i].p_status == PFREE || players[i].p_stats.st_tticks==0)
	    continue;
	switch(mode) {
	case 'd':
	    printf("  %2s: %-16s %-8s   %2s %6.2f %6d %7d %6d %6d\n",
		players[i].p_mapchars, players[i].p_name,
		players[i].p_login,
		ships[players[i].p_ship.s_type],
		players[i].p_kills,
		players[i].p_damage,
		players[i].p_shield,
		players[i].p_armies,
		players[i].p_fuel);
	    teams[players[i].p_team]++;
	    break;
	default:
	    strncpy(monitor, players[i].p_monitor, 16);
	    monitor[16]=0;
	    printf("  %2s: %-16s %-16s   %2s %7.2f %7.2f %7.2f %7.2f\n", 
		players[i].p_mapchars, players[i].p_name, 
		monitor, ships[players[i].p_ship.s_type],
		bombingRating(players+i),
		planetRating(players+i),
		offenseRating(players+i),
		defenseRating(players+i));
	    teams[players[i].p_team]++;
	    break;
	}
    }
    switch(mode) {
    case 'd':
	puts(linestr[1]);
	printf("  Feds: %d   Roms: %d   Kli: %d   Ori: %d\n", 
	    teams[FED], teams[ROM], teams[KLI], teams[ORI]);
	puts(linestr[1]);
	break;
    default:
	puts(linestr[0]);
	printf("  Feds: %d   Roms: %d   Kli: %d   Ori: %d\n", 
	    teams[FED], teams[ROM], teams[KLI], teams[ORI]);
	puts(linestr[0]);
    }

    exit(0);
}

/*
 * ATM: show the #of people playing.  Updated every 30 seconds.
 */
showcount()
{
    extern int errno;
    int	shmemKey = PKEY;
    int	shmid;
    struct memory	*sharedMemory;
    int i, count, oldcount;

    oldcount = -1;
    while (1) {
	errno = 0;
	shmid = shmget(shmemKey, 0, 0);
	if (shmid < 0) {
	    if (errno != ENOENT) {
		perror("shmget");
		exit(1);
	    }
	    shmid = shmget(shmemKey, 0, 0);
	    if (shmid < 0) {
		count = 0;
		goto nobody;
	    }
	}
	sharedMemory = (struct memory *) shmat(shmid, 0, 0);
	if (sharedMemory == (struct memory *) -1) {
	    perror("shared memory");
	    exit(1);
	}
	players = sharedMemory->players;

	count = 0;
	for (i=0; i<MAXPLAYER; i++) {
	    if (players[i].p_status != PFREE && players[i].p_stats.st_tticks!=0)
		count++;
	}
	if (shmdt(sharedMemory) < 0) {
	    perror("shmdt");
	    exit(1);
	}
nobody:

	if (count != oldcount) {
	    printf("\rNow %2d", count);  fflush(stdout);
	    oldcount = count;
	}
	sleep(30);
    }
}

